
#include "sector.h"
#include "paramSimul.h"
#include "evaluationConfigurerations.h"
#include "ue.h"
#include "const.h"


/*===================================================================
FUNCTION: Sector::Sector()

AUTHOR: SJ

DESCRIPTION: Sector Object Constructor

NOTES: Currently, nothing is done when a sector is instantiated

===================================================================*/

Sector::Sector()
{
	//pschedulewrite = new SCHEDULE_DECISION[num_rbs_in_1ms];
	cqis = NULL;

}

/*===================================================================
FUNCTION: Sector::~Sector()

AUTHOR: SJ

DESCRIPTION: Sector Object Destructor

NOTES: Currently, nothing is done

===================================================================*/
Sector::~Sector()
{
	delete pschedulewrite;
}

/*===================================================================
FUNCTION: Sector::Configure( int )

AUTHOR: SJ

DESCRIPTION: Sector object gets to know the index of itself

NOTES: More functionality may be added

===================================================================*/
void Sector::initialization(int sector_idx)
{
	ue_in_control.clear();
	
	_sector_idx = sector_idx;

	if (num_TRxP_per_site == 3)
	{
		alpha = (sector_idx % 3 * 120 + 30)*PI / 180;
		
	}
	else
	{
		alpha = 0;
	}
	
	beta = (bs_antenna.mechanic_tilt - 90) * PI / 180;
	gamma = 0;
}

/*===================================================================
FUNCTION: Sector::RR_scheduling()

AUTHOR: GJ

DESCRIPTION:

OUTPUT: pschedulewrite[rb_idx].ue_selected

NOTES: More functionality may be added

===================================================================*/

void Sector::RR_scheduling()
{
	ramdom_selection_UE();
	determin_MCS();

}


void Sector::ramdom_selection_UE()
{
	int rs_size = num_rbs_in_1ms;
	int num_rs = num_rbs_in_1ms / rs_size;

	SCHEDULE_DECISION *decision;
	decision = new SCHEDULE_DECISION[num_rs];

	if (num_rs > int(ue_in_control.size()))
	{
		num_rs = ue_in_control.size();
	}


	for (int rs_idx = 0; rs_idx < num_rs; rs_idx++)
	{
		decision[rs_idx].ue_selected = -1;
	}

	for (int rs_idx = 0; rs_idx < num_rs; rs_idx++)
	{
		int selected_ue;
		bool end_decision;
		do
		{
			selected_ue = (int)(Rand.u()*ue_in_control.size());

			end_decision = true;
			for (int check_idx = 0; check_idx < rs_idx; check_idx++)
			{
				if (decision[check_idx].ue_selected == ue_in_control[selected_ue])
					end_decision = false;
			}
		} while (!end_decision);
		decision[rs_idx].ue_selected = ue_in_control[selected_ue];
	}


	for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
	{
		if ((int)rb_idx / rs_size < num_rs)
		{
			pschedulewrite[rb_idx] = decision[(int)(rb_idx / rs_size)];
		}
		else
		{
			pschedulewrite[rb_idx] = pschedulewrite[rb_idx - 1];
		}
	}

	/*
	for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
	{
	cout << pschedulewrite[rb_idx].ue_selected << endl;
	}
	*/

	delete decision;
}
void Sector::determin_MCS()
{
	if (cqis == NULL)
	{
		cqis = new int *[ue_in_control.size()];
		for (int ue_idx = 0; ue_idx < int(ue_in_control.size()); ue_idx++)
		{
			cqis[ue_idx] = new int[num_rbs_in_1ms];
		}
	}



	for (int ue_idx = 0; ue_idx < int(ue_in_control.size()); ue_idx++)
	{
		int min_cqi = NUM_CQI_MCS_TYPES;
		// MIN CQI within UE setting for each frequency
		for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
		{
			if (pschedulewrite[rb_idx].ue_selected == ue_in_control[ue_idx])
			{
				if (min_cqi > cqis[ue_idx][rb_idx])
				{
					min_cqi = cqis[ue_idx][rb_idx];
				}
			}
		}

		// Set MIN CQI Scheduling for each frequency
		for (int rb_idx = 0; rb_idx < num_rbs_in_1ms; rb_idx++)
		{
			if (pschedulewrite[rb_idx].ue_selected == ue_in_control[ue_idx])
			{
				pschedulewrite[rb_idx].mcs_selected = min_cqi;
			}
		}
	}
}
void Sector::PF_scheduling()
{


}



