#include <iostream>

#include "objectBSSectorUe.h"
#include "paramSimul.h"
#include "evaluationConfigurerations.h"
#include "const.h"
#include "struct.h"


using namespace std;

void Set_antenna_parameter();
void Generate_2D_DFT_beam_precoder();
void Generate_bs_2D_DFT_beam_precoder();
void Generate_ue_2D_DFT_beam_precoder();
complex <double> Generate_vertical_virtualization_weight(double theta, double dv, double k, double K);
complex <double> Generate_horizontal_virtualization_weight(double theta, double phi, double dH, double l, double L);


void Set_antenna_configuration(void)
{
	Set_antenna_parameter();

	Generate_2D_DFT_beam_precoder();	
}

void Set_antenna_parameter()
{
	if (test_environment == InH_eMBB_A)
	{
		bs_antenna.element_gain_dBi = 5.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 32;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 4;
		bs_antenna.N = 4;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 4;
		bs_antenna.Np = 4;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.5;


		if (num_TRxP_per_site == 3)
		{
			bs_antenna.mechanic_tilt = 110;
		}
		else
		{
			bs_antenna.mechanic_tilt = 180;
		}
		bs_antenna.electronic_tilt = 90;
	
		bs_antenna.polarization[0] = 45;
		bs_antenna.polarization[1] = -45;

		ue_antenna.M = 1;
		ue_antenna.N = 2;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 2;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;


		bs_analog_beam.zenith_steering_angle[0] = bs_antenna.electronic_tilt / 180.0 * PI;

		bs_analog_beam.num_steering_azimuth_angle = 1;
		bs_analog_beam.num_steering_zenith_angle = 1;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;
	}
	else if (test_environment == InH_eMBB_B)
	{
		bs_antenna.element_gain_dBi = 5.0;//dBi
		ue_antenna.element_gain_dBi = 5.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 4;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 2;
		bs_antenna.Np = 2;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.5;

		if (num_TRxP_per_site == 3)
		{
			bs_antenna.mechanic_tilt = 110;
		}
		else
		{
			bs_antenna.mechanic_tilt = 180;
		}

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 2;
		ue_antenna.N = 4;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 2;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = lamda_0 * 0.5;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;


		bs_analog_beam.num_steering_azimuth_angle = 4;
		bs_analog_beam.num_steering_zenith_angle = 2;
		bs_analog_beam.azimuth_steering_angle[0] = -3.0 / 8.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -1.0 / 8.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = 1.0 / 8.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = 3.0 / 8.0 * PI;
		
		bs_analog_beam.zenith_steering_angle[0] = 1.0/ 4.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 3.0 / 4.0 * PI;


		ue_analog_beam.num_steering_azimuth_angle = 4;
		ue_analog_beam.num_steering_zenith_angle = 2;
		ue_analog_beam.azimuth_steering_angle[0] = -3.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[1] = -1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[2] = 1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[3] = 3.0 / 8.0 * PI;
		
		ue_analog_beam.zenith_steering_angle[0] = 1.0 / 4.0 * PI;
		ue_analog_beam.zenith_steering_angle[1] = 3.0 / 4.0 * PI;



	}
	else if (test_environment == InH_eMBB_C)
	{

		bs_antenna.element_gain_dBi = 5.0;//dBi
		ue_antenna.element_gain_dBi = 5.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 8;
		bs_antenna.N = 16;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 2;
		bs_antenna.Np = 2;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.5;

		
		if (num_TRxP_per_site == 3)
		{
			bs_antenna.mechanic_tilt = 110;
		}
		else
		{
			bs_antenna.mechanic_tilt = 180;
		}

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 2;
		ue_antenna.N = 4;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 2;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = lamda_0 * 0.5;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;

		bs_analog_beam.num_steering_azimuth_angle = 8;
		bs_analog_beam.num_steering_zenith_angle = 4;

		bs_analog_beam.azimuth_steering_angle[0] = -7.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = -3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = -1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[4] = 1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[5] = 3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[6] = 5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[7] = 7.0 / 16.0 * PI;
		
		bs_analog_beam.zenith_steering_angle[0] = 1.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 3.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[2] = 5.0 / 8.0* PI;
		bs_analog_beam.zenith_steering_angle[3] = 7.0 / 8.0 * PI;


		ue_analog_beam.num_steering_azimuth_angle = 4;
		ue_analog_beam.num_steering_zenith_angle = 2;

		ue_analog_beam.azimuth_steering_angle[0] = -3.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[1] = -1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[2] = 1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[3] = 3.0 / 8.0 * PI;

		ue_analog_beam.zenith_steering_angle[0] = 1.0 / 4.0 * PI;
		ue_analog_beam.zenith_steering_angle[1] = 3.0 / 4.0 * PI;


	}
	else if (test_environment == Dense_Urban_eMBB_A)
	{

		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 4;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 8;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 2;
		bs_antenna.Np = 1;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.8;

		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 1;
		ue_antenna.N = 2;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;


		 //빔포밍 고정
		/*bs_analog_beam.num_steering_azimuth_angle = 1;
		bs_analog_beam.num_steering_zenith_angle = 1;*/
		bs_analog_beam.num_steering_azimuth_angle = 6;
		bs_analog_beam.num_steering_zenith_angle = 2;


		bs_analog_beam.azimuth_steering_angle[0] = -5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = -1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = 1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[4] = 3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[5] = 5.0 / 16.0 * PI;

		bs_analog_beam.zenith_steering_angle[0] = 5.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 7.0 / 8.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;
	}
	else if (test_environment == Dense_Urban_eMBB_B)
	{

		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 5.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 4;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 2;
		bs_antenna.Ng = 2;

		bs_antenna.Mp = 1;
		bs_antenna.Np = 1;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.5;

		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 2;
		ue_antenna.N = 4;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 2;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = lamda_0 * 0.5;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;


		bs_analog_beam.num_steering_azimuth_angle = 6;
		bs_analog_beam.num_steering_zenith_angle = 2;

		bs_analog_beam.azimuth_steering_angle[0] = -5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = -1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = 1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[4] = 3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[5] = 5.0 / 16.0 * PI;

		bs_analog_beam.zenith_steering_angle[0] = 5.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 7.0 / 8.0 * PI;


		ue_analog_beam.num_steering_azimuth_angle = 4;
		ue_analog_beam.num_steering_zenith_angle = 2;

		ue_analog_beam.azimuth_steering_angle[0] = -3.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[1] = -1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[2] = 1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[3] = 3.0 / 8.0 * PI;
		
		ue_analog_beam.zenith_steering_angle[0] = 1.0 / 4.0 * PI;
		ue_analog_beam.zenith_steering_angle[1] = 3.0 / 4.0 * PI;


	}
	else if (test_environment == Dense_Urban_eMBB_C_Macro)
	{

		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 4;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 8;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 2;
		bs_antenna.Np = 1;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.8;

		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 1;
		ue_antenna.N = 2;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;


		bs_analog_beam.num_steering_azimuth_angle = 6;
		bs_analog_beam.num_steering_zenith_angle = 2;

		bs_analog_beam.azimuth_steering_angle[0] = -5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = -1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = 1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[4] = 3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[5] = 5.0 / 16.0 * PI;

		bs_analog_beam.zenith_steering_angle[0] = 5.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 7.0 / 8.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;
	}
	else if (test_environment == Dense_Urban_eMBB_C_Micro)
	{
		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 5.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 4;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 2;
		bs_antenna.Ng = 2;

		bs_antenna.Mp = 1;
		bs_antenna.Np = 1;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.5;

		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 2;
		ue_antenna.N = 4;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 2;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;
		ue_antenna.dV = lamda_0 * 0.5;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;

		
		bs_analog_beam.num_steering_azimuth_angle = 6;
		bs_analog_beam.num_steering_zenith_angle = 2;

		bs_analog_beam.azimuth_steering_angle[0] = -5.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[1] = -3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[2] = -1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[3] = 1.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[4] = 3.0 / 16.0 * PI;
		bs_analog_beam.azimuth_steering_angle[5] = 5.0 / 16.0 * PI;

		bs_analog_beam.zenith_steering_angle[0] = 5.0 / 8.0 * PI;
		bs_analog_beam.zenith_steering_angle[1] = 7.0 / 8.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 4;
		ue_analog_beam.num_steering_zenith_angle = 2;

		ue_analog_beam.azimuth_steering_angle[0] = -3.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[1] = -1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[2] =  1.0 / 8.0 * PI;
		ue_analog_beam.azimuth_steering_angle[3] =  3.0 / 8.0 * PI;
		ue_analog_beam.zenith_steering_angle[0] =   1.0 / 4.0 * PI;
		ue_analog_beam.zenith_steering_angle[1] =   3.0 / 4.0 * PI;
	}
	else if (test_environment == Rural_eMBB_A)
	{
		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 2;

		bs_antenna.M = 8;
		bs_antenna.N = 4;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 1;
		bs_antenna.Np = 4;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.8;

		bs_antenna.electronic_tilt = 100;
		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 1;
		ue_antenna.N = 1;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = 0;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;



		bs_analog_beam.num_steering_azimuth_angle = 1;
		bs_analog_beam.num_steering_zenith_angle = 1;

		bs_analog_beam.zenith_steering_angle[0] = 100.0 / 180.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;
	}
	else if (test_environment == Rural_eMBB_B)
	{
		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 16;
		ue_antenna.num_txru = 4;

		bs_antenna.M = 8;
		bs_antenna.N = 8;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 1;
		bs_antenna.Np = 8;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.8;

		bs_antenna.electronic_tilt = 100;
		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 1;
		ue_antenna.N = 2;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;

		bs_analog_beam.num_steering_azimuth_angle = 1;
		bs_analog_beam.num_steering_zenith_angle = 1;

		bs_analog_beam.zenith_steering_angle[0] = 100.0 / 180.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;

	}
	else if (test_environment == Rural_eMBB_C)
	{
		bs_antenna.element_gain_dBi = 8.0;//dBi
		ue_antenna.element_gain_dBi = 0.0;//dBi

		bs_antenna.num_txru = 8;
		ue_antenna.num_txru = 4;


		bs_antenna.M = 8;
		bs_antenna.N = 4;
		bs_antenna.P = 2;
		bs_antenna.Mg = 1;
		bs_antenna.Ng = 1;

		bs_antenna.Mp = 1;
		bs_antenna.Np = 4;
		bs_antenna.dH = lamda_0 * 0.5;
		bs_antenna.dV = lamda_0 * 0.8;

		bs_antenna.electronic_tilt = 96;
		bs_antenna.mechanic_tilt = 90;

		bs_antenna.polarization[0] = -45;
		bs_antenna.polarization[1] = 45;

		ue_antenna.M = 1;
		ue_antenna.N = 2;
		ue_antenna.P = 2;
		ue_antenna.Mg = 1;
		ue_antenna.Ng = 1;

		ue_antenna.Mp = 1;
		ue_antenna.Np = 1;
		ue_antenna.dH = lamda_0 * 0.5;;
		ue_antenna.dV = 0;

		bs_antenna.polarization[0] = 0;
		bs_antenna.polarization[1] = 90;

		bs_analog_beam.num_steering_azimuth_angle = 1;
		bs_analog_beam.num_steering_zenith_angle = 1;

		bs_analog_beam.zenith_steering_angle[0] = bs_antenna.electronic_tilt / 180.0 * PI;

		ue_analog_beam.num_steering_azimuth_angle = 0;
		ue_analog_beam.num_steering_zenith_angle = 0;
	}

	bs_antenna.K = bs_antenna.M / bs_antenna.Mp;
	bs_antenna.L = bs_antenna.N / bs_antenna.Np;
	ue_antenna.K = ue_antenna.M / ue_antenna.Mp;
	ue_antenna.L = ue_antenna.N / ue_antenna.Np;

}


void Generate_2D_DFT_beam_precoder()
{

	Generate_bs_2D_DFT_beam_precoder();

	Generate_ue_2D_DFT_beam_precoder();

}


void Generate_bs_2D_DFT_beam_precoder()
{
	complex <double> w[4][8];   //[num_vertical steering angle][num element in a port] (maximum)
	complex <double> v[4][8][8];//[num_vertical steering angle][num_horizontal sterring angle][num element in a port]  (maximum)

	for (int zenith_idx = 0; zenith_idx < bs_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{
		for (int k = 0; k < bs_antenna.K; k++) // vertical element
		{
			w[zenith_idx][k] = Generate_vertical_virtualization_weight(bs_analog_beam.zenith_steering_angle[zenith_idx], bs_antenna.dV, double(k), double(bs_antenna.K));

		}
	}

	for (int zenith_idx = 0; zenith_idx < bs_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{
		for (int azimuth_idx = 0; azimuth_idx < bs_analog_beam.num_steering_azimuth_angle; azimuth_idx++)
		{
			for (int l = 0; l < bs_antenna.L; l++) // horizontal element
			{
				v[zenith_idx][azimuth_idx][l] = Generate_horizontal_virtualization_weight(bs_analog_beam.zenith_steering_angle[zenith_idx], bs_analog_beam.azimuth_steering_angle[azimuth_idx], bs_antenna.dH, double(l), double(bs_antenna.L));
			}
		}
	}



	for (int zenith_idx = 0; zenith_idx<bs_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{
		for (int azimuth_idx = 0; azimuth_idx<bs_analog_beam.num_steering_azimuth_angle; azimuth_idx++)
		{
			for (int k = 0; k < bs_antenna.K; k++) // vertical element
			{
				for (int l = 0; l < bs_antenna.L; l++) // horizontal element
				{
					bs_analog_beam.virtualization_weight[zenith_idx][azimuth_idx][k][l] = w[zenith_idx][k]*v[zenith_idx][azimuth_idx][l];
				}
			}
		}
	}

}

void Generate_ue_2D_DFT_beam_precoder()
{
	complex <double> w[2][2];
	complex <double> v[2][4][4];

	for (int zenith_idx = 0; zenith_idx < ue_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{
		for (int k = 0; k < ue_antenna.K; k++) // vertical element
		{
			w[zenith_idx][k] = Generate_vertical_virtualization_weight(ue_analog_beam.zenith_steering_angle[zenith_idx], ue_antenna.dV, double(k), double(ue_antenna.K));
		}
	}
	
	
	for (int zenith_idx = 0; zenith_idx < ue_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{

		for (int azimuth_idx = 0; azimuth_idx < ue_analog_beam.num_steering_azimuth_angle; azimuth_idx++)
		{
			for (int l = 0; l < ue_antenna.L; l++) // horizontal element
			{
				v[zenith_idx][azimuth_idx][l] = Generate_horizontal_virtualization_weight(ue_analog_beam.zenith_steering_angle[zenith_idx], ue_analog_beam.azimuth_steering_angle[azimuth_idx], ue_antenna.dH, double(l), double(ue_antenna.L));
			}
		}
	}


	for (int zenith_idx = 0; zenith_idx<ue_analog_beam.num_steering_zenith_angle; zenith_idx++)
	{
		for (int azimuth_idx = 0; azimuth_idx<ue_analog_beam.num_steering_azimuth_angle; azimuth_idx++)
		{
			for (int k = 0; k < ue_antenna.K; k++) // vertical element
			{
				for (int l = 0; l < ue_antenna.L; l++) // horizontal element
				{
					ue_analog_beam.virtualization_weight[zenith_idx][azimuth_idx][k][l] = w[zenith_idx][k] * v[zenith_idx][azimuth_idx][l];
				}
			}		
		}
	}

}




complex <double> Generate_vertical_virtualization_weight(double theta, double dv, double k, double K)
{
	complex <double> j(0.0, 1.0);
	complex <double> w;

	w = 1.0 / sqrt(double(K))*exp(-j * 2.0 * PI / lamda_0 * k * dv*cos(theta));
	
	return w;
}


complex <double> Generate_horizontal_virtualization_weight(double theta,double phi, double dH, double l, double L)
{
	complex <double> j(0.0, 1.0);
	complex <double> v;

	v = 1.0 / sqrt(double(L))*exp(-j * 2.0 * PI / lamda_0 * l * dH * sin(theta) * sin(phi));

	return v;
}
